package sun.net.www;

import java.net.URLConnection;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.io.FileOutputStream;
import java.util.StringTokenizer;
import java.io.IOException;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.File;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Hashtable;
import java.net.FileNameMap;

public class MimeTable implements FileNameMap
{
    private Hashtable<String, MimeEntry> entries;
    private Hashtable<String, MimeEntry> extensionMap;
    private static String tempFileTemplate;
    private static final String filePreamble = "sun.net.www MIME content-types table";
    private static final String fileMagic = "#sun.net.www MIME content-types table";
    protected static String[] mailcapLocations;
    
    MimeTable() {
        this.entries = new Hashtable();
        this.extensionMap = new Hashtable();
        this.load();
    }
    
    public static MimeTable getDefaultTable() {
        return MimeTable.DefaultInstanceHolder.defaultInstance;
    }
    
    public static FileNameMap loadTable() {
        return getDefaultTable();
    }
    
    public synchronized int getSize() {
        return this.entries.size();
    }
    
    @Override
    public synchronized String getContentTypeFor(String s) {
        MimeEntry byFileName = this.findByFileName(s);
        if (byFileName != null) {
            return byFileName.getType();
        }
        return null;
    }
    
    public synchronized void add(MimeEntry mimeEntry) {
        this.entries.put(mimeEntry.getType(), mimeEntry);
        String[] extensions = mimeEntry.getExtensions();
        if (extensions == null) {
            return;
        }
        for (int i = 0; i < extensions.length; ++i) {
            this.extensionMap.put(extensions[i], mimeEntry);
        }
        return;
    }
    
    public synchronized MimeEntry remove(String key) {
        return this.remove((MimeEntry)this.entries.get(key));
    }
    
    public synchronized MimeEntry remove(MimeEntry mimeEntry) {
        String[] extensions = mimeEntry.getExtensions();
        if (extensions != null) {
            for (int i = 0; i < extensions.length; ++i) {
                this.extensionMap.remove(extensions[i]);
            }
        }
        return (MimeEntry)this.entries.remove(mimeEntry.getType());
    }
    
    public synchronized MimeEntry find(String key) {
        MimeEntry mimeEntry = (MimeEntry)this.entries.get(key);
        if (mimeEntry == null) {
            Enumeration<MimeEntry> elements = this.entries.elements();
            while (elements.hasMoreElements()) {
                MimeEntry mimeEntry2 = (MimeEntry)elements.nextElement();
                if (mimeEntry2.matches(key)) {
                    return mimeEntry2;
                }
                continue;
            }
            return mimeEntry;
        }
        return mimeEntry;
    }
    
    public MimeEntry findByFileName(String substring) {
        String lowerCase = "";
        int lastIndex = substring.lastIndexOf(35);
        if (lastIndex > 0) {
            substring = substring.substring(0, lastIndex - 1);
        }
        int max = Math.max(Math.max(substring.lastIndexOf(46), substring.lastIndexOf(47)), substring.lastIndexOf(63));
        if (max != -1 && substring.charAt(max) == '.') {
            lowerCase = substring.substring(max).toLowerCase();
        }
        return this.findByExt(lowerCase);
    }
    
    public synchronized MimeEntry findByExt(String key) {
        return (MimeEntry)this.extensionMap.get(key);
    }
    
    public synchronized MimeEntry findByDescription(String s) {
        Enumeration<MimeEntry> elements = this.elements();
        while (elements.hasMoreElements()) {
            MimeEntry mimeEntry = (MimeEntry)elements.nextElement();
            if (s.equals(mimeEntry.getDescription())) {
                return mimeEntry;
            }
            continue;
        }
        return this.find(s);
    }
    
    String getTempFileTemplate() {
        return MimeTable.tempFileTemplate;
    }
    
    public synchronized Enumeration<MimeEntry> elements() {
        return this.entries.elements();
    }
    
    public synchronized void load() {
        Properties properties = new Properties();
        File file = null;
        try {
            String property = System.getProperty("content.types.user.table");
            if (property != null) {
                file = new File(property);
                if (!file.exists()) {
                    file = new File(System.getProperty("java.home") + File.separator + "lib" + File.separator + "content-types.properties");
                }
            }
            else {
                file = new File(System.getProperty("java.home") + File.separator + "lib" + File.separator + "content-types.properties");
            }
            BufferedInputStream inStream = new BufferedInputStream(new FileInputStream(file));
            properties.load(inStream);
            inStream.close();
        }
        catch (IOException ex) {
            System.err.println("Warning: default mime table not found: " + file.getPath());
            return;
        }
        this.parse(properties);
    }
    
    void parse(Properties properties) {
        String tempFileTemplate = (String)properties.get("temp.file.template");
        if (tempFileTemplate != null) {
            properties.remove("temp.file.template");
            MimeTable.tempFileTemplate = tempFileTemplate;
        }
        Enumeration<Object> propertyNames = properties.propertyNames();
        while (propertyNames.hasMoreElements()) {
            String key = (String)propertyNames.nextElement();
            this.parse(key, properties.getProperty(key));
        }
    }
    
    void parse(String s, String str) {
        MimeEntry mimeEntry = new MimeEntry(s);
        StringTokenizer stringTokenizer = new StringTokenizer(str, ";");
        while (stringTokenizer.hasMoreTokens()) {
            this.parse(stringTokenizer.nextToken(), mimeEntry);
        }
        this.add(mimeEntry);
    }
    
    void parse(String str, MimeEntry mimeEntry) {
        String trim = null;
        String trim2 = null;
        int n = 0;
        StringTokenizer stringTokenizer = new StringTokenizer(str, "=");
        while (stringTokenizer.hasMoreTokens()) {
            if (n != 0) {
                trim2 = stringTokenizer.nextToken().trim();
            }
            else {
                trim = stringTokenizer.nextToken().trim();
                n = 1;
            }
        }
        this.fill(mimeEntry, trim, trim2);
    }
    
    void fill(MimeEntry mimeEntry, String anotherString, String s) {
        if ("description".equalsIgnoreCase(anotherString)) {
            mimeEntry.setDescription(s);
        }
        else if ("action".equalsIgnoreCase(anotherString)) {
            mimeEntry.setAction(this.getActionCode(s));
        }
        else if ("application".equalsIgnoreCase(anotherString)) {
            mimeEntry.setCommand(s);
        }
        else if ("icon".equalsIgnoreCase(anotherString)) {
            mimeEntry.setImageFileName(s);
        }
        else if ("file_extensions".equalsIgnoreCase(anotherString)) {
            mimeEntry.setExtensions(s);
        }
    }
    
    String[] getExtensions(String str) {
        StringTokenizer stringTokenizer = new StringTokenizer(str, ",");
        int countTokens = stringTokenizer.countTokens();
        String[] array = new String[countTokens];
        for (int i = 0; i < countTokens; ++i) {
            array[i] = stringTokenizer.nextToken();
        }
        return array;
    }
    
    int getActionCode(String s) {
        int i = 0;
        while (i < MimeEntry.actionKeywords.length) {
            if (s.equalsIgnoreCase(MimeEntry.actionKeywords[i])) {
                return i;
            }
            ++i;
        }
        return 0;
    }
    
    public synchronized boolean save(String property) {
        if (property == null) {
            property = System.getProperty("user.home" + File.separator + "lib" + File.separator + "content-types.properties");
        }
        return this.saveAsProperties(new File(property));
    }
    
    public Properties getAsProperties() {
        Properties properties = new Properties();
        Enumeration<MimeEntry> elements = this.elements();
        while (elements.hasMoreElements()) {
            MimeEntry mimeEntry = (MimeEntry)elements.nextElement();
            properties.put(mimeEntry.getType(), mimeEntry.toProperty());
        }
        return properties;
    }
    
    protected boolean saveAsProperties(File file) {
        FileOutputStream fileOutputStream = null;
        try {
            fileOutputStream = new FileOutputStream(file);
            Properties asProperties = this.getAsProperties();
            asProperties.put("temp.file.template", MimeTable.tempFileTemplate);
            String property = System.getProperty("user.name");
            if (property != null) {
                asProperties.store(fileOutputStream, "sun.net.www MIME content-types table" + ("; customized for " + property));
            }
            else {
                asProperties.store(fileOutputStream, "sun.net.www MIME content-types table");
            }
        }
        catch (IOException ex) {
            ex.printStackTrace();
            return false;
        }
        finally {
            if (fileOutputStream != null) {
                try {
                    fileOutputStream.close();
                }
                catch (IOException ex2) {}
            }
        }
        return true;
    }
    
    static {
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                MimeTable.tempFileTemplate = System.getProperty("content.types.temp.file.template", "/tmp/%s");
                MimeTable.mailcapLocations = new String[] { System.getProperty("user.mailcap"), System.getProperty("user.home") + "/.mailcap", "/etc/mailcap", "/usr/etc/mailcap", "/usr/local/etc/mailcap", System.getProperty("hotjava.home", "/usr/local/hotjava") + "/lib/mailcap" };
                return null;
            }
        });
    }
    
    private static class DefaultInstanceHolder
    {
        static final MimeTable defaultInstance;
        
        static MimeTable getDefaultInstance() {
            return (MimeTable)AccessController.doPrivileged(new PrivilegedAction<MimeTable>() {
                @Override
                public MimeTable run() {
                    MimeTable fileNameMap = new MimeTable();
                    URLConnection.setFileNameMap(fileNameMap);
                    return fileNameMap;
                }
            });
        }
        
        static {
            defaultInstance = getDefaultInstance();
        }
    }
}
